# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .apps import (
    AppsResource,
    AsyncAppsResource,
    AppsResourceWithRawResponse,
    AsyncAppsResourceWithRawResponse,
    AppsResourceWithStreamingResponse,
    AsyncAppsResourceWithStreamingResponse,
)
from .presets import (
    PresetsResource,
    AsyncPresetsResource,
    PresetsResourceWithRawResponse,
    AsyncPresetsResourceWithRawResponse,
    PresetsResourceWithStreamingResponse,
    AsyncPresetsResourceWithStreamingResponse,
)
from .meetings import (
    MeetingsResource,
    AsyncMeetingsResource,
    MeetingsResourceWithRawResponse,
    AsyncMeetingsResourceWithRawResponse,
    MeetingsResourceWithStreamingResponse,
    AsyncMeetingsResourceWithStreamingResponse,
)
from .sessions import (
    SessionsResource,
    AsyncSessionsResource,
    SessionsResourceWithRawResponse,
    AsyncSessionsResourceWithRawResponse,
    SessionsResourceWithStreamingResponse,
    AsyncSessionsResourceWithStreamingResponse,
)
from .webhooks import (
    WebhooksResource,
    AsyncWebhooksResource,
    WebhooksResourceWithRawResponse,
    AsyncWebhooksResourceWithRawResponse,
    WebhooksResourceWithStreamingResponse,
    AsyncWebhooksResourceWithStreamingResponse,
)
from ..._compat import cached_property
from .analytics import (
    AnalyticsResource,
    AsyncAnalyticsResource,
    AnalyticsResourceWithRawResponse,
    AsyncAnalyticsResourceWithRawResponse,
    AnalyticsResourceWithStreamingResponse,
    AsyncAnalyticsResourceWithStreamingResponse,
)
from .recordings import (
    RecordingsResource,
    AsyncRecordingsResource,
    RecordingsResourceWithRawResponse,
    AsyncRecordingsResourceWithRawResponse,
    RecordingsResourceWithStreamingResponse,
    AsyncRecordingsResourceWithStreamingResponse,
)
from ..._resource import SyncAPIResource, AsyncAPIResource
from .livestreams import (
    LivestreamsResource,
    AsyncLivestreamsResource,
    LivestreamsResourceWithRawResponse,
    AsyncLivestreamsResourceWithRawResponse,
    LivestreamsResourceWithStreamingResponse,
    AsyncLivestreamsResourceWithStreamingResponse,
)
from .active_session import (
    ActiveSessionResource,
    AsyncActiveSessionResource,
    ActiveSessionResourceWithRawResponse,
    AsyncActiveSessionResourceWithRawResponse,
    ActiveSessionResourceWithStreamingResponse,
    AsyncActiveSessionResourceWithStreamingResponse,
)

__all__ = ["RealtimeKitResource", "AsyncRealtimeKitResource"]


class RealtimeKitResource(SyncAPIResource):
    @cached_property
    def apps(self) -> AppsResource:
        return AppsResource(self._client)

    @cached_property
    def meetings(self) -> MeetingsResource:
        return MeetingsResource(self._client)

    @cached_property
    def presets(self) -> PresetsResource:
        return PresetsResource(self._client)

    @cached_property
    def sessions(self) -> SessionsResource:
        return SessionsResource(self._client)

    @cached_property
    def recordings(self) -> RecordingsResource:
        return RecordingsResource(self._client)

    @cached_property
    def webhooks(self) -> WebhooksResource:
        return WebhooksResource(self._client)

    @cached_property
    def active_session(self) -> ActiveSessionResource:
        return ActiveSessionResource(self._client)

    @cached_property
    def livestreams(self) -> LivestreamsResource:
        return LivestreamsResource(self._client)

    @cached_property
    def analytics(self) -> AnalyticsResource:
        return AnalyticsResource(self._client)

    @cached_property
    def with_raw_response(self) -> RealtimeKitResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return RealtimeKitResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> RealtimeKitResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return RealtimeKitResourceWithStreamingResponse(self)


class AsyncRealtimeKitResource(AsyncAPIResource):
    @cached_property
    def apps(self) -> AsyncAppsResource:
        return AsyncAppsResource(self._client)

    @cached_property
    def meetings(self) -> AsyncMeetingsResource:
        return AsyncMeetingsResource(self._client)

    @cached_property
    def presets(self) -> AsyncPresetsResource:
        return AsyncPresetsResource(self._client)

    @cached_property
    def sessions(self) -> AsyncSessionsResource:
        return AsyncSessionsResource(self._client)

    @cached_property
    def recordings(self) -> AsyncRecordingsResource:
        return AsyncRecordingsResource(self._client)

    @cached_property
    def webhooks(self) -> AsyncWebhooksResource:
        return AsyncWebhooksResource(self._client)

    @cached_property
    def active_session(self) -> AsyncActiveSessionResource:
        return AsyncActiveSessionResource(self._client)

    @cached_property
    def livestreams(self) -> AsyncLivestreamsResource:
        return AsyncLivestreamsResource(self._client)

    @cached_property
    def analytics(self) -> AsyncAnalyticsResource:
        return AsyncAnalyticsResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncRealtimeKitResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncRealtimeKitResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncRealtimeKitResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncRealtimeKitResourceWithStreamingResponse(self)


class RealtimeKitResourceWithRawResponse:
    def __init__(self, realtime_kit: RealtimeKitResource) -> None:
        self._realtime_kit = realtime_kit

    @cached_property
    def apps(self) -> AppsResourceWithRawResponse:
        return AppsResourceWithRawResponse(self._realtime_kit.apps)

    @cached_property
    def meetings(self) -> MeetingsResourceWithRawResponse:
        return MeetingsResourceWithRawResponse(self._realtime_kit.meetings)

    @cached_property
    def presets(self) -> PresetsResourceWithRawResponse:
        return PresetsResourceWithRawResponse(self._realtime_kit.presets)

    @cached_property
    def sessions(self) -> SessionsResourceWithRawResponse:
        return SessionsResourceWithRawResponse(self._realtime_kit.sessions)

    @cached_property
    def recordings(self) -> RecordingsResourceWithRawResponse:
        return RecordingsResourceWithRawResponse(self._realtime_kit.recordings)

    @cached_property
    def webhooks(self) -> WebhooksResourceWithRawResponse:
        return WebhooksResourceWithRawResponse(self._realtime_kit.webhooks)

    @cached_property
    def active_session(self) -> ActiveSessionResourceWithRawResponse:
        return ActiveSessionResourceWithRawResponse(self._realtime_kit.active_session)

    @cached_property
    def livestreams(self) -> LivestreamsResourceWithRawResponse:
        return LivestreamsResourceWithRawResponse(self._realtime_kit.livestreams)

    @cached_property
    def analytics(self) -> AnalyticsResourceWithRawResponse:
        return AnalyticsResourceWithRawResponse(self._realtime_kit.analytics)


class AsyncRealtimeKitResourceWithRawResponse:
    def __init__(self, realtime_kit: AsyncRealtimeKitResource) -> None:
        self._realtime_kit = realtime_kit

    @cached_property
    def apps(self) -> AsyncAppsResourceWithRawResponse:
        return AsyncAppsResourceWithRawResponse(self._realtime_kit.apps)

    @cached_property
    def meetings(self) -> AsyncMeetingsResourceWithRawResponse:
        return AsyncMeetingsResourceWithRawResponse(self._realtime_kit.meetings)

    @cached_property
    def presets(self) -> AsyncPresetsResourceWithRawResponse:
        return AsyncPresetsResourceWithRawResponse(self._realtime_kit.presets)

    @cached_property
    def sessions(self) -> AsyncSessionsResourceWithRawResponse:
        return AsyncSessionsResourceWithRawResponse(self._realtime_kit.sessions)

    @cached_property
    def recordings(self) -> AsyncRecordingsResourceWithRawResponse:
        return AsyncRecordingsResourceWithRawResponse(self._realtime_kit.recordings)

    @cached_property
    def webhooks(self) -> AsyncWebhooksResourceWithRawResponse:
        return AsyncWebhooksResourceWithRawResponse(self._realtime_kit.webhooks)

    @cached_property
    def active_session(self) -> AsyncActiveSessionResourceWithRawResponse:
        return AsyncActiveSessionResourceWithRawResponse(self._realtime_kit.active_session)

    @cached_property
    def livestreams(self) -> AsyncLivestreamsResourceWithRawResponse:
        return AsyncLivestreamsResourceWithRawResponse(self._realtime_kit.livestreams)

    @cached_property
    def analytics(self) -> AsyncAnalyticsResourceWithRawResponse:
        return AsyncAnalyticsResourceWithRawResponse(self._realtime_kit.analytics)


class RealtimeKitResourceWithStreamingResponse:
    def __init__(self, realtime_kit: RealtimeKitResource) -> None:
        self._realtime_kit = realtime_kit

    @cached_property
    def apps(self) -> AppsResourceWithStreamingResponse:
        return AppsResourceWithStreamingResponse(self._realtime_kit.apps)

    @cached_property
    def meetings(self) -> MeetingsResourceWithStreamingResponse:
        return MeetingsResourceWithStreamingResponse(self._realtime_kit.meetings)

    @cached_property
    def presets(self) -> PresetsResourceWithStreamingResponse:
        return PresetsResourceWithStreamingResponse(self._realtime_kit.presets)

    @cached_property
    def sessions(self) -> SessionsResourceWithStreamingResponse:
        return SessionsResourceWithStreamingResponse(self._realtime_kit.sessions)

    @cached_property
    def recordings(self) -> RecordingsResourceWithStreamingResponse:
        return RecordingsResourceWithStreamingResponse(self._realtime_kit.recordings)

    @cached_property
    def webhooks(self) -> WebhooksResourceWithStreamingResponse:
        return WebhooksResourceWithStreamingResponse(self._realtime_kit.webhooks)

    @cached_property
    def active_session(self) -> ActiveSessionResourceWithStreamingResponse:
        return ActiveSessionResourceWithStreamingResponse(self._realtime_kit.active_session)

    @cached_property
    def livestreams(self) -> LivestreamsResourceWithStreamingResponse:
        return LivestreamsResourceWithStreamingResponse(self._realtime_kit.livestreams)

    @cached_property
    def analytics(self) -> AnalyticsResourceWithStreamingResponse:
        return AnalyticsResourceWithStreamingResponse(self._realtime_kit.analytics)


class AsyncRealtimeKitResourceWithStreamingResponse:
    def __init__(self, realtime_kit: AsyncRealtimeKitResource) -> None:
        self._realtime_kit = realtime_kit

    @cached_property
    def apps(self) -> AsyncAppsResourceWithStreamingResponse:
        return AsyncAppsResourceWithStreamingResponse(self._realtime_kit.apps)

    @cached_property
    def meetings(self) -> AsyncMeetingsResourceWithStreamingResponse:
        return AsyncMeetingsResourceWithStreamingResponse(self._realtime_kit.meetings)

    @cached_property
    def presets(self) -> AsyncPresetsResourceWithStreamingResponse:
        return AsyncPresetsResourceWithStreamingResponse(self._realtime_kit.presets)

    @cached_property
    def sessions(self) -> AsyncSessionsResourceWithStreamingResponse:
        return AsyncSessionsResourceWithStreamingResponse(self._realtime_kit.sessions)

    @cached_property
    def recordings(self) -> AsyncRecordingsResourceWithStreamingResponse:
        return AsyncRecordingsResourceWithStreamingResponse(self._realtime_kit.recordings)

    @cached_property
    def webhooks(self) -> AsyncWebhooksResourceWithStreamingResponse:
        return AsyncWebhooksResourceWithStreamingResponse(self._realtime_kit.webhooks)

    @cached_property
    def active_session(self) -> AsyncActiveSessionResourceWithStreamingResponse:
        return AsyncActiveSessionResourceWithStreamingResponse(self._realtime_kit.active_session)

    @cached_property
    def livestreams(self) -> AsyncLivestreamsResourceWithStreamingResponse:
        return AsyncLivestreamsResourceWithStreamingResponse(self._realtime_kit.livestreams)

    @cached_property
    def analytics(self) -> AsyncAnalyticsResourceWithStreamingResponse:
        return AsyncAnalyticsResourceWithStreamingResponse(self._realtime_kit.analytics)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from typing_extensions import Literal, Required, TypeAlias, TypedDict

from ....._types import SequenceNotStr

__all__ = ["AccountMappingCreateParams", "AuthRequirements", "AuthRequirementsUnionMember0", "AuthRequirementsType"]


class AccountMappingCreateParams(TypedDict, total=False):
    account_id: Required[str]

    auth_requirements: Required[AuthRequirements]


class AuthRequirementsUnionMember0(TypedDict, total=False):
    allowed_microsoft_organizations: Required[SequenceNotStr[str]]

    type: Required[Literal["Org"]]


class AuthRequirementsType(TypedDict, total=False):
    type: Required[Literal["NoAuth"]]


AuthRequirements: TypeAlias = Union[AuthRequirementsUnionMember0, AuthRequirementsType]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, Optional, cast

import httpx

from ..._types import Body, Query, Headers, NoneType, NotGiven, not_given
from ..._utils import path_template
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._wrappers import ResultWrapper
from .credentials import (
    CredentialsResource,
    AsyncCredentialsResource,
    CredentialsResourceWithRawResponse,
    AsyncCredentialsResourceWithRawResponse,
    CredentialsResourceWithStreamingResponse,
    AsyncCredentialsResourceWithStreamingResponse,
)
from ..._base_client import make_request_options
from .maintenance_configs import (
    MaintenanceConfigsResource,
    AsyncMaintenanceConfigsResource,
    MaintenanceConfigsResourceWithRawResponse,
    AsyncMaintenanceConfigsResourceWithRawResponse,
    MaintenanceConfigsResourceWithStreamingResponse,
    AsyncMaintenanceConfigsResourceWithStreamingResponse,
)
from .namespaces.namespaces import (
    NamespacesResource,
    AsyncNamespacesResource,
    NamespacesResourceWithRawResponse,
    AsyncNamespacesResourceWithRawResponse,
    NamespacesResourceWithStreamingResponse,
    AsyncNamespacesResourceWithStreamingResponse,
)
from ...types.r2_data_catalog.r2_data_catalog_get_response import R2DataCatalogGetResponse
from ...types.r2_data_catalog.r2_data_catalog_list_response import R2DataCatalogListResponse
from ...types.r2_data_catalog.r2_data_catalog_enable_response import R2DataCatalogEnableResponse

__all__ = ["R2DataCatalogResource", "AsyncR2DataCatalogResource"]


class R2DataCatalogResource(SyncAPIResource):
    @cached_property
    def maintenance_configs(self) -> MaintenanceConfigsResource:
        return MaintenanceConfigsResource(self._client)

    @cached_property
    def credentials(self) -> CredentialsResource:
        return CredentialsResource(self._client)

    @cached_property
    def namespaces(self) -> NamespacesResource:
        return NamespacesResource(self._client)

    @cached_property
    def with_raw_response(self) -> R2DataCatalogResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return R2DataCatalogResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> R2DataCatalogResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return R2DataCatalogResourceWithStreamingResponse(self)

    def list(
        self,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[R2DataCatalogListResponse]:
        """
        Returns a list of R2 buckets that have been enabled as Apache Iceberg catalogs
        for the specified account. Each catalog represents an R2 bucket configured to
        store Iceberg metadata and data files.

        Args:
          account_id: Use this to identify the account.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get(
            path_template("/accounts/{account_id}/r2-catalog", account_id=account_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[R2DataCatalogListResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[R2DataCatalogListResponse]], ResultWrapper[R2DataCatalogListResponse]),
        )

    def disable(
        self,
        bucket_name: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """Disable an R2 bucket as a catalog.

        This operation deactivates the catalog but
        preserves existing metadata and data files. The catalog can be re-enabled later.

        Args:
          account_id: Use this to identify the account.

          bucket_name: Specifies the R2 bucket name.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not bucket_name:
            raise ValueError(f"Expected a non-empty value for `bucket_name` but received {bucket_name!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._post(
            path_template(
                "/accounts/{account_id}/r2-catalog/{bucket_name}/disable",
                account_id=account_id,
                bucket_name=bucket_name,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    def enable(
        self,
        bucket_name: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[R2DataCatalogEnableResponse]:
        """Enable an R2 bucket as an Apache Iceberg catalog.

        This operation creates the
        necessary catalog infrastructure and activates the bucket for storing Iceberg
        metadata and data files.

        Args:
          account_id: Use this to identify the account.

          bucket_name: Specifies the R2 bucket name.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not bucket_name:
            raise ValueError(f"Expected a non-empty value for `bucket_name` but received {bucket_name!r}")
        return self._post(
            path_template(
                "/accounts/{account_id}/r2-catalog/{bucket_name}/enable", account_id=account_id, bucket_name=bucket_name
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[R2DataCatalogEnableResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[R2DataCatalogEnableResponse]], ResultWrapper[R2DataCatalogEnableResponse]),
        )

    def get(
        self,
        bucket_name: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[R2DataCatalogGetResponse]:
        """
        Retrieve detailed information about a specific R2 catalog by bucket name.
        Returns catalog status, maintenance configuration, and credential status.

        Args:
          account_id: Use this to identify the account.

          bucket_name: Specifies the R2 bucket name.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not bucket_name:
            raise ValueError(f"Expected a non-empty value for `bucket_name` but received {bucket_name!r}")
        return self._get(
            path_template(
                "/accounts/{account_id}/r2-catalog/{bucket_name}", account_id=account_id, bucket_name=bucket_name
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[R2DataCatalogGetResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[R2DataCatalogGetResponse]], ResultWrapper[R2DataCatalogGetResponse]),
        )


class AsyncR2DataCatalogResource(AsyncAPIResource):
    @cached_property
    def maintenance_configs(self) -> AsyncMaintenanceConfigsResource:
        return AsyncMaintenanceConfigsResource(self._client)

    @cached_property
    def credentials(self) -> AsyncCredentialsResource:
        return AsyncCredentialsResource(self._client)

    @cached_property
    def namespaces(self) -> AsyncNamespacesResource:
        return AsyncNamespacesResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncR2DataCatalogResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncR2DataCatalogResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncR2DataCatalogResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncR2DataCatalogResourceWithStreamingResponse(self)

    async def list(
        self,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[R2DataCatalogListResponse]:
        """
        Returns a list of R2 buckets that have been enabled as Apache Iceberg catalogs
        for the specified account. Each catalog represents an R2 bucket configured to
        store Iceberg metadata and data files.

        Args:
          account_id: Use this to identify the account.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._get(
            path_template("/accounts/{account_id}/r2-catalog", account_id=account_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[R2DataCatalogListResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[R2DataCatalogListResponse]], ResultWrapper[R2DataCatalogListResponse]),
        )

    async def disable(
        self,
        bucket_name: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """Disable an R2 bucket as a catalog.

        This operation deactivates the catalog but
        preserves existing metadata and data files. The catalog can be re-enabled later.

        Args:
          account_id: Use this to identify the account.

          bucket_name: Specifies the R2 bucket name.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not bucket_name:
            raise ValueError(f"Expected a non-empty value for `bucket_name` but received {bucket_name!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._post(
            path_template(
                "/accounts/{account_id}/r2-catalog/{bucket_name}/disable",
                account_id=account_id,
                bucket_name=bucket_name,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    async def enable(
        self,
        bucket_name: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[R2DataCatalogEnableResponse]:
        """Enable an R2 bucket as an Apache Iceberg catalog.

        This operation creates the
        necessary catalog infrastructure and activates the bucket for storing Iceberg
        metadata and data files.

        Args:
          account_id: Use this to identify the account.

          bucket_name: Specifies the R2 bucket name.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not bucket_name:
            raise ValueError(f"Expected a non-empty value for `bucket_name` but received {bucket_name!r}")
        return await self._post(
            path_template(
                "/accounts/{account_id}/r2-catalog/{bucket_name}/enable", account_id=account_id, bucket_name=bucket_name
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[R2DataCatalogEnableResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[R2DataCatalogEnableResponse]], ResultWrapper[R2DataCatalogEnableResponse]),
        )

    async def get(
        self,
        bucket_name: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[R2DataCatalogGetResponse]:
        """
        Retrieve detailed information about a specific R2 catalog by bucket name.
        Returns catalog status, maintenance configuration, and credential status.

        Args:
          account_id: Use this to identify the account.

          bucket_name: Specifies the R2 bucket name.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not bucket_name:
            raise ValueError(f"Expected a non-empty value for `bucket_name` but received {bucket_name!r}")
        return await self._get(
            path_template(
                "/accounts/{account_id}/r2-catalog/{bucket_name}", account_id=account_id, bucket_name=bucket_name
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[R2DataCatalogGetResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[R2DataCatalogGetResponse]], ResultWrapper[R2DataCatalogGetResponse]),
        )


class R2DataCatalogResourceWithRawResponse:
    def __init__(self, r2_data_catalog: R2DataCatalogResource) -> None:
        self._r2_data_catalog = r2_data_catalog

        self.list = to_raw_response_wrapper(
            r2_data_catalog.list,
        )
        self.disable = to_raw_response_wrapper(
            r2_data_catalog.disable,
        )
        self.enable = to_raw_response_wrapper(
            r2_data_catalog.enable,
        )
        self.get = to_raw_response_wrapper(
            r2_data_catalog.get,
        )

    @cached_property
    def maintenance_configs(self) -> MaintenanceConfigsResourceWithRawResponse:
        return MaintenanceConfigsResourceWithRawResponse(self._r2_data_catalog.maintenance_configs)

    @cached_property
    def credentials(self) -> CredentialsResourceWithRawResponse:
        return CredentialsResourceWithRawResponse(self._r2_data_catalog.credentials)

    @cached_property
    def namespaces(self) -> NamespacesResourceWithRawResponse:
        return NamespacesResourceWithRawResponse(self._r2_data_catalog.namespaces)


class AsyncR2DataCatalogResourceWithRawResponse:
    def __init__(self, r2_data_catalog: AsyncR2DataCatalogResource) -> None:
        self._r2_data_catalog = r2_data_catalog

        self.list = async_to_raw_response_wrapper(
            r2_data_catalog.list,
        )
        self.disable = async_to_raw_response_wrapper(
            r2_data_catalog.disable,
        )
        self.enable = async_to_raw_response_wrapper(
            r2_data_catalog.enable,
        )
        self.get = async_to_raw_response_wrapper(
            r2_data_catalog.get,
        )

    @cached_property
    def maintenance_configs(self) -> AsyncMaintenanceConfigsResourceWithRawResponse:
        return AsyncMaintenanceConfigsResourceWithRawResponse(self._r2_data_catalog.maintenance_configs)

    @cached_property
    def credentials(self) -> AsyncCredentialsResourceWithRawResponse:
        return AsyncCredentialsResourceWithRawResponse(self._r2_data_catalog.credentials)

    @cached_property
    def namespaces(self) -> AsyncNamespacesResourceWithRawResponse:
        return AsyncNamespacesResourceWithRawResponse(self._r2_data_catalog.namespaces)


class R2DataCatalogResourceWithStreamingResponse:
    def __init__(self, r2_data_catalog: R2DataCatalogResource) -> None:
        self._r2_data_catalog = r2_data_catalog

        self.list = to_streamed_response_wrapper(
            r2_data_catalog.list,
        )
        self.disable = to_streamed_response_wrapper(
            r2_data_catalog.disable,
        )
        self.enable = to_streamed_response_wrapper(
            r2_data_catalog.enable,
        )
        self.get = to_streamed_response_wrapper(
            r2_data_catalog.get,
        )

    @cached_property
    def maintenance_configs(self) -> MaintenanceConfigsResourceWithStreamingResponse:
        return MaintenanceConfigsResourceWithStreamingResponse(self._r2_data_catalog.maintenance_configs)

    @cached_property
    def credentials(self) -> CredentialsResourceWithStreamingResponse:
        return CredentialsResourceWithStreamingResponse(self._r2_data_catalog.credentials)

    @cached_property
    def namespaces(self) -> NamespacesResourceWithStreamingResponse:
        return NamespacesResourceWithStreamingResponse(self._r2_data_catalog.namespaces)


class AsyncR2DataCatalogResourceWithStreamingResponse:
    def __init__(self, r2_data_catalog: AsyncR2DataCatalogResource) -> None:
        self._r2_data_catalog = r2_data_catalog

        self.list = async_to_streamed_response_wrapper(
            r2_data_catalog.list,
        )
        self.disable = async_to_streamed_response_wrapper(
            r2_data_catalog.disable,
        )
        self.enable = async_to_streamed_response_wrapper(
            r2_data_catalog.enable,
        )
        self.get = async_to_streamed_response_wrapper(
            r2_data_catalog.get,
        )

    @cached_property
    def maintenance_configs(self) -> AsyncMaintenanceConfigsResourceWithStreamingResponse:
        return AsyncMaintenanceConfigsResourceWithStreamingResponse(self._r2_data_catalog.maintenance_configs)

    @cached_property
    def credentials(self) -> AsyncCredentialsResourceWithStreamingResponse:
        return AsyncCredentialsResourceWithStreamingResponse(self._r2_data_catalog.credentials)

    @cached_property
    def namespaces(self) -> AsyncNamespacesResourceWithStreamingResponse:
        return AsyncNamespacesResourceWithStreamingResponse(self._r2_data_catalog.namespaces)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

from ...._types import SequenceNotStr

__all__ = ["ApprovalGroupParam"]


class ApprovalGroupParam(TypedDict, total=False):
    """A group of email addresses that can approve a temporary authentication request."""

    approvals_needed: Required[float]
    """The number of approvals needed to obtain access."""

    email_addresses: SequenceNotStr[str]
    """A list of emails that can approve the access request."""

    email_list_uuid: str
    """The UUID of an re-usable email list."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Union, Optional
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["DNSRecord"]


class DNSRecord(BaseModel):
    """List of records needed to enable an Email Routing zone."""

    content: Optional[str] = None
    """DNS record content."""

    name: Optional[str] = None
    """DNS record name (or @ for the zone apex)."""

    priority: Optional[float] = None
    """Required for MX, SRV and URI records.

    Unused by other record types. Records with lower priorities are preferred.
    """

    ttl: Union[float, Literal[1], None] = None
    """Time to live, in seconds, of the DNS record.

    Must be between 60 and 86400, or 1 for 'automatic'.
    """

    type: Optional[
        Literal[
            "A",
            "AAAA",
            "CNAME",
            "HTTPS",
            "TXT",
            "SRV",
            "LOC",
            "MX",
            "NS",
            "CERT",
            "DNSKEY",
            "DS",
            "NAPTR",
            "SMIMEA",
            "SSHFP",
            "SVCB",
            "TLSA",
            "URI",
        ]
    ] = None
    """DNS record type."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from datetime import datetime
from typing_extensions import Literal, Required, Annotated, TypeAlias, TypedDict

from ..._utils import PropertyInfo
from .pcap_filter_param import PCAPFilterParam

__all__ = ["PCAPCreateParams", "MagicVisibilityPCAPsPCAPsRequestSimple", "MagicVisibilityPCAPsPCAPsRequestFull"]


class MagicVisibilityPCAPsPCAPsRequestSimple(TypedDict, total=False):
    account_id: Required[str]
    """Identifier."""

    packet_limit: Required[float]
    """The limit of packets contained in a packet capture."""

    system: Required[Literal["magic-transit"]]
    """The system used to collect packet captures."""

    time_limit: Required[float]
    """The packet capture duration in seconds."""

    type: Required[Literal["simple", "full"]]
    """The type of packet capture.

    `Simple` captures sampled packets, and `full` captures entire payloads and
    non-sampled packets.
    """

    filter_v1: PCAPFilterParam
    """The packet capture filter. When this field is empty, all packets are captured."""

    offset_time: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]
    """The RFC 3339 offset timestamp from which to query backwards for packets.

    Must be within the last 24h. When this field is empty, defaults to time of
    request.
    """


class MagicVisibilityPCAPsPCAPsRequestFull(TypedDict, total=False):
    account_id: Required[str]
    """Identifier."""

    colo_name: Required[str]
    """The name of the data center used for the packet capture.

    This can be a specific colo (ord02) or a multi-colo name (ORD). This field only
    applies to `full` packet captures.
    """

    destination_conf: Required[str]
    """The full URI for the bucket. This field only applies to `full` packet captures."""

    system: Required[Literal["magic-transit"]]
    """The system used to collect packet captures."""

    time_limit: Required[float]
    """The packet capture duration in seconds."""

    type: Required[Literal["simple", "full"]]
    """The type of packet capture.

    `Simple` captures sampled packets, and `full` captures entire payloads and
    non-sampled packets.
    """

    byte_limit: float
    """The maximum number of bytes to capture.

    This field only applies to `full` packet captures.
    """

    filter_v1: PCAPFilterParam
    """The packet capture filter. When this field is empty, all packets are captured."""

    packet_limit: float
    """The limit of packets contained in a packet capture."""


PCAPCreateParams: TypeAlias = Union[MagicVisibilityPCAPsPCAPsRequestSimple, MagicVisibilityPCAPsPCAPsRequestFull]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .site import Site as Site
from .rum_rule import RUMRule as RUMRule
from .rule_create_params import RuleCreateParams as RuleCreateParams
from .rule_list_response import RuleListResponse as RuleListResponse
from .rule_update_params import RuleUpdateParams as RuleUpdateParams
from .rule_delete_response import RuleDeleteResponse as RuleDeleteResponse
from .site_info_list_params import SiteInfoListParams as SiteInfoListParams
from .rule_bulk_create_params import RuleBulkCreateParams as RuleBulkCreateParams
from .site_info_create_params import SiteInfoCreateParams as SiteInfoCreateParams
from .site_info_update_params import SiteInfoUpdateParams as SiteInfoUpdateParams
from .rule_bulk_create_response import RuleBulkCreateResponse as RuleBulkCreateResponse
from .site_info_delete_response import SiteInfoDeleteResponse as SiteInfoDeleteResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Iterable, Optional
from typing_extensions import Literal, Required, TypedDict

__all__ = ["RecordingStartTrackRecordingParams", "Layers", "LayersOutput", "LayersOutputStorageConfig"]


class RecordingStartTrackRecordingParams(TypedDict, total=False):
    account_id: Required[str]
    """The account identifier tag."""

    layers: Required[Dict[str, Layers]]

    meeting_id: Required[str]
    """ID of the meeting to record."""

    max_seconds: float
    """Maximum seconds this recording should be active for (beta)"""


class LayersOutputStorageConfig(TypedDict, total=False):
    type: Required[Literal["aws", "azure", "digitalocean", "gcs", "sftp"]]
    """Type of storage media."""

    access_key: str
    """Access key of the storage medium.

    Access key is not required for the `gcs` storage media type.

    Note that this field is not readable by clients, only writeable.
    """

    auth_method: Literal["KEY", "PASSWORD"]
    """Authentication method used for "sftp" type storage medium"""

    bucket: str
    """Name of the storage medium's bucket."""

    host: str
    """SSH destination server host for SFTP type storage medium"""

    password: str
    """
    SSH destination server password for SFTP type storage medium when auth_method is
    "PASSWORD". If auth_method is "KEY", this specifies the password for the ssh
    private key.
    """

    path: str
    """Path relative to the bucket root at which the recording will be placed."""

    port: float
    """SSH destination server port for SFTP type storage medium"""

    private_key: str
    """
    Private key used to login to destination SSH server for SFTP type storage
    medium, when auth_method used is "KEY"
    """

    region: str
    """Region of the storage medium."""

    secret: str
    """Secret key of the storage medium.

    Similar to `access_key`, it is only writeable by clients, not readable.
    """

    username: str
    """SSH destination server username for SFTP type storage medium"""


class LayersOutput(TypedDict, total=False):
    storage_config: Optional[LayersOutputStorageConfig]

    type: Literal["REALTIMEKIT_BUCKET", "STORAGE_CONFIG"]
    """The type of output destination this layer is being exported to."""


class Layers(TypedDict, total=False):
    file_name_prefix: str
    """A file name prefix to apply for files generated from this layer"""

    outputs: Iterable[LayersOutput]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, Annotated, TypedDict

from ..._utils import PropertyInfo

__all__ = ["BucketListParams"]


class BucketListParams(TypedDict, total=False):
    account_id: Required[str]
    """Account ID."""

    cursor: str
    """Pagination cursor received during the last List Buckets call.

    R2 buckets are paginated using cursors instead of page numbers.
    """

    direction: Literal["asc", "desc"]
    """Direction to order buckets."""

    name_contains: str
    """Bucket names to filter by.

    Only buckets with this phrase in their name will be returned.
    """

    order: Literal["name"]
    """Field to order buckets by."""

    per_page: float
    """Maximum number of buckets to return in a single call."""

    start_after: str
    """Bucket name to start searching after. Buckets are ordered lexicographically."""

    jurisdiction: Annotated[Literal["default", "eu", "fedramp"], PropertyInfo(alias="cf-r2-jurisdiction")]
    """Jurisdiction where objects in this bucket are guaranteed to be stored."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict
from typing_extensions import Required, TypedDict

__all__ = ["RuleBulkEditParams"]


class RuleBulkEditParams(TypedDict, total=False):
    account_id: Required[str]

    new_priorities: Required[Dict[str, int]]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

from .skip_configuration_param import SkipConfigurationParam

__all__ = ["ContextAwarenessParam"]


class ContextAwarenessParam(TypedDict, total=False):
    """
    Scan the context of predefined entries to only return matches surrounded by keywords.
    """

    enabled: Required[bool]
    """
    If true, scan the context of predefined entries to only return matches
    surrounded by keywords.
    """

    skip: Required[SkipConfigurationParam]
    """Content types to exclude from context analysis and return all matches."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

from ...._types import SequenceNotStr

__all__ = ["SCIMConfigAuthenticationOauth2Param"]


class SCIMConfigAuthenticationOauth2Param(TypedDict, total=False):
    """
    Attributes for configuring OAuth 2 authentication scheme for SCIM provisioning to an application.
    """

    authorization_url: Required[str]
    """URL used to generate the auth code used during token generation."""

    client_id: Required[str]
    """
    Client ID used to authenticate when generating a token for authenticating with
    the remote SCIM service.
    """

    client_secret: Required[str]
    """
    Secret used to authenticate when generating a token for authenticating with the
    remove SCIM service.
    """

    scheme: Required[Literal["oauth2"]]
    """The authentication scheme to use when making SCIM requests to this application."""

    token_url: Required[str]
    """
    URL used to generate the token used to authenticate with the remote SCIM
    service.
    """

    scopes: SequenceNotStr[str]
    """
    The authorization scopes to request when generating the token used to
    authenticate with the remove SCIM service.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, Optional, cast

import httpx

from ...._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ...._utils import path_template, maybe_transform, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._wrappers import ResultWrapper
from ...._base_client import make_request_options
from ....types.intel.indicator_feeds import permission_create_params, permission_delete_params
from ....types.intel.indicator_feeds.permission_list_response import PermissionListResponse
from ....types.intel.indicator_feeds.permission_create_response import PermissionCreateResponse
from ....types.intel.indicator_feeds.permission_delete_response import PermissionDeleteResponse

__all__ = ["PermissionsResource", "AsyncPermissionsResource"]


class PermissionsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> PermissionsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return PermissionsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> PermissionsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return PermissionsResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        account_id: str,
        account_tag: str | Omit = omit,
        feed_id: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[PermissionCreateResponse]:
        """
        Grants access permissions for a custom threat indicator feed to other accounts.

        Args:
          account_id: Identifier

          account_tag: The Cloudflare account tag of the account to change permissions on

          feed_id: The ID of the feed to add/remove permissions on

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._put(
            path_template("/accounts/{account_id}/intel/indicator-feeds/permissions/add", account_id=account_id),
            body=maybe_transform(
                {
                    "account_tag": account_tag,
                    "feed_id": feed_id,
                },
                permission_create_params.PermissionCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[PermissionCreateResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[PermissionCreateResponse]], ResultWrapper[PermissionCreateResponse]),
        )

    def list(
        self,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[PermissionListResponse]:
        """
        Lists current access permissions for custom threat indicator feeds.

        Args:
          account_id: Identifier

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get(
            path_template("/accounts/{account_id}/intel/indicator-feeds/permissions/view", account_id=account_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[PermissionListResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[PermissionListResponse]], ResultWrapper[PermissionListResponse]),
        )

    def delete(
        self,
        *,
        account_id: str,
        account_tag: str | Omit = omit,
        feed_id: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[PermissionDeleteResponse]:
        """
        Revokes access permissions for a custom threat indicator feed.

        Args:
          account_id: Identifier

          account_tag: The Cloudflare account tag of the account to change permissions on

          feed_id: The ID of the feed to add/remove permissions on

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._put(
            path_template("/accounts/{account_id}/intel/indicator-feeds/permissions/remove", account_id=account_id),
            body=maybe_transform(
                {
                    "account_tag": account_tag,
                    "feed_id": feed_id,
                },
                permission_delete_params.PermissionDeleteParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[PermissionDeleteResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[PermissionDeleteResponse]], ResultWrapper[PermissionDeleteResponse]),
        )


class AsyncPermissionsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncPermissionsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncPermissionsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncPermissionsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncPermissionsResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        account_id: str,
        account_tag: str | Omit = omit,
        feed_id: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[PermissionCreateResponse]:
        """
        Grants access permissions for a custom threat indicator feed to other accounts.

        Args:
          account_id: Identifier

          account_tag: The Cloudflare account tag of the account to change permissions on

          feed_id: The ID of the feed to add/remove permissions on

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._put(
            path_template("/accounts/{account_id}/intel/indicator-feeds/permissions/add", account_id=account_id),
            body=await async_maybe_transform(
                {
                    "account_tag": account_tag,
                    "feed_id": feed_id,
                },
                permission_create_params.PermissionCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[PermissionCreateResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[PermissionCreateResponse]], ResultWrapper[PermissionCreateResponse]),
        )

    async def list(
        self,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[PermissionListResponse]:
        """
        Lists current access permissions for custom threat indicator feeds.

        Args:
          account_id: Identifier

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._get(
            path_template("/accounts/{account_id}/intel/indicator-feeds/permissions/view", account_id=account_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[PermissionListResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[PermissionListResponse]], ResultWrapper[PermissionListResponse]),
        )

    async def delete(
        self,
        *,
        account_id: str,
        account_tag: str | Omit = omit,
        feed_id: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[PermissionDeleteResponse]:
        """
        Revokes access permissions for a custom threat indicator feed.

        Args:
          account_id: Identifier

          account_tag: The Cloudflare account tag of the account to change permissions on

          feed_id: The ID of the feed to add/remove permissions on

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._put(
            path_template("/accounts/{account_id}/intel/indicator-feeds/permissions/remove", account_id=account_id),
            body=await async_maybe_transform(
                {
                    "account_tag": account_tag,
                    "feed_id": feed_id,
                },
                permission_delete_params.PermissionDeleteParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[PermissionDeleteResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[PermissionDeleteResponse]], ResultWrapper[PermissionDeleteResponse]),
        )


class PermissionsResourceWithRawResponse:
    def __init__(self, permissions: PermissionsResource) -> None:
        self._permissions = permissions

        self.create = to_raw_response_wrapper(
            permissions.create,
        )
        self.list = to_raw_response_wrapper(
            permissions.list,
        )
        self.delete = to_raw_response_wrapper(
            permissions.delete,
        )


class AsyncPermissionsResourceWithRawResponse:
    def __init__(self, permissions: AsyncPermissionsResource) -> None:
        self._permissions = permissions

        self.create = async_to_raw_response_wrapper(
            permissions.create,
        )
        self.list = async_to_raw_response_wrapper(
            permissions.list,
        )
        self.delete = async_to_raw_response_wrapper(
            permissions.delete,
        )


class PermissionsResourceWithStreamingResponse:
    def __init__(self, permissions: PermissionsResource) -> None:
        self._permissions = permissions

        self.create = to_streamed_response_wrapper(
            permissions.create,
        )
        self.list = to_streamed_response_wrapper(
            permissions.list,
        )
        self.delete = to_streamed_response_wrapper(
            permissions.delete,
        )


class AsyncPermissionsResourceWithStreamingResponse:
    def __init__(self, permissions: AsyncPermissionsResource) -> None:
        self._permissions = permissions

        self.create = async_to_streamed_response_wrapper(
            permissions.create,
        )
        self.list = async_to_streamed_response_wrapper(
            permissions.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            permissions.delete,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List
from typing_extensions import TypeAlias

from .verification import Verification

__all__ = ["VerificationGetResponse"]

VerificationGetResponse: TypeAlias = List[Verification]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .os_get_params import OSGetParams as OSGetParams
from .os_get_response import OSGetResponse as OSGetResponse
from .bot_class_get_params import BotClassGetParams as BotClassGetParams
from .ip_version_get_params import IPVersionGetParams as IPVersionGetParams
from .bot_class_get_response import BotClassGetResponse as BotClassGetResponse
from .device_type_get_params import DeviceTypeGetParams as DeviceTypeGetParams
from .http_method_get_params import HTTPMethodGetParams as HTTPMethodGetParams
from .tls_version_get_params import TLSVersionGetParams as TLSVersionGetParams
from .ip_version_get_response import IPVersionGetResponse as IPVersionGetResponse
from .device_type_get_response import DeviceTypeGetResponse as DeviceTypeGetResponse
from .http_method_get_response import HTTPMethodGetResponse as HTTPMethodGetResponse
from .http_protocol_get_params import HTTPProtocolGetParams as HTTPProtocolGetParams
from .tls_version_get_response import TLSVersionGetResponse as TLSVersionGetResponse
from .browser_family_get_params import BrowserFamilyGetParams as BrowserFamilyGetParams
from .http_protocol_get_response import HTTPProtocolGetResponse as HTTPProtocolGetResponse
from .browser_family_get_response import BrowserFamilyGetResponse as BrowserFamilyGetResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

from ....._types import SequenceNotStr

__all__ = ["GroupListParams"]


class GroupListParams(TypedDict, total=False):
    account_id: Required[str]
    """Identifier."""

    cf_resource_id: SequenceNotStr[str]
    """
    The unique Cloudflare-generated Id of the SCIM Group resource; also known as the
    "Id". Pass once for a single lookup (`?cf_resource_id=A`) or repeat the
    parameter (`?cf_resource_id=A&cf_resource_id=B`) to look up multiple groups in
    one request, up to 50 values. Mutually exclusive with `idp_resource_id`, `name`,
    `search_contains`, and `search_starts_with`.
    """

    idp_resource_id: SequenceNotStr[str]
    """
    The IdP-generated Id of the SCIM Group resource; also known as the "external
    Id". Pass once for a single lookup (`?idp_resource_id=A`) or repeat the
    parameter (`?idp_resource_id=A&idp_resource_id=B`) to look up multiple groups in
    one request, up to 50 values. Mutually exclusive with `cf_resource_id`, `name`,
    `search_contains`, and `search_starts_with`.
    """

    name: str
    """The display name of the SCIM Group resource."""

    page: int
    """Page number of results."""

    per_page: int
    """Number of results per page."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from ...._models import BaseModel

__all__ = ["SCIMConfigMapping", "Operations"]


class Operations(BaseModel):
    """Whether or not this mapping applies to creates, updates, or deletes."""

    create: Optional[bool] = None
    """Whether or not this mapping applies to create (POST) operations."""

    delete: Optional[bool] = None
    """Whether or not this mapping applies to DELETE operations."""

    update: Optional[bool] = None
    """Whether or not this mapping applies to update (PATCH/PUT) operations."""


class SCIMConfigMapping(BaseModel):
    """
    Transformations and filters applied to resources before they are provisioned in the remote SCIM service.
    """

    schema_: str = FieldInfo(alias="schema")
    """Which SCIM resource type this mapping applies to."""

    enabled: Optional[bool] = None
    """Whether or not this mapping is enabled."""

    filter: Optional[str] = None
    """
    A
    [SCIM filter expression](https://datatracker.ietf.org/doc/html/rfc7644#section-3.4.2.2)
    that matches resources that should be provisioned to this application.
    """

    operations: Optional[Operations] = None
    """Whether or not this mapping applies to creates, updates, or deletes."""

    strictness: Optional[Literal["strict", "passthrough"]] = None
    """
    The level of adherence to outbound resource schemas when provisioning to this
    mapping. ‘Strict’ removes unknown values, while ‘passthrough’ passes unknown
    values to the target.
    """

    transform_jsonata: Optional[str] = None
    """
    A [JSONata](https://jsonata.org/) expression that transforms the resource before
    provisioning it in the application.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .summary_get_response import SummaryGetResponse as SummaryGetResponse
from .behaviour_get_response import BehaviourGetResponse as BehaviourGetResponse
from .behaviour_update_params import BehaviourUpdateParams as BehaviourUpdateParams
from .integration_get_response import IntegrationGetResponse as IntegrationGetResponse
from .behaviour_update_response import BehaviourUpdateResponse as BehaviourUpdateResponse
from .integration_create_params import IntegrationCreateParams as IntegrationCreateParams
from .integration_list_response import IntegrationListResponse as IntegrationListResponse
from .integration_update_params import IntegrationUpdateParams as IntegrationUpdateParams
from .integration_create_response import IntegrationCreateResponse as IntegrationCreateResponse
from .integration_update_response import IntegrationUpdateResponse as IntegrationUpdateResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .scan_result import ScanResult as ScanResult
from .config_edit_params import ConfigEditParams as ConfigEditParams
from .result_get_response import ResultGetResponse as ResultGetResponse
from .config_create_params import ConfigCreateParams as ConfigCreateParams
from .config_edit_response import ConfigEditResponse as ConfigEditResponse
from .config_list_response import ConfigListResponse as ConfigListResponse
from .config_create_response import ConfigCreateResponse as ConfigCreateResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["Invite"]


class Invite(BaseModel):
    invited_member_id: Optional[str] = None
    """ID of the user to add to the organization."""

    organization_id: str
    """ID of the organization the user will be added to."""

    id: Optional[str] = None
    """Invite identifier tag."""

    expires_on: Optional[datetime] = None
    """When the invite is no longer active."""

    invited_by: Optional[str] = None
    """The email address of the user who created the invite."""

    invited_member_email: Optional[str] = None
    """Email address of the user to add to the organization."""

    invited_on: Optional[datetime] = None
    """When the invite was sent."""

    organization_is_enforcing_twofactor: Optional[bool] = None

    organization_name: Optional[str] = None
    """Organization name."""

    roles: Optional[List[str]] = None
    """List of role names the membership has for this account."""

    status: Optional[Literal["pending", "accepted", "rejected", "expired"]] = None
    """Current status of the invitation."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, Optional, cast

import httpx

from ..._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ..._utils import path_template, maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._wrappers import ResultWrapper
from ...pagination import SyncSinglePage, AsyncSinglePage
from ..._base_client import AsyncPaginator, make_request_options
from ...types.stream import watermark_create_params
from ...types.stream.watermark import Watermark
from ...types.stream.watermark_delete_response import WatermarkDeleteResponse

__all__ = ["WatermarksResource", "AsyncWatermarksResource"]


class WatermarksResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> WatermarksResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return WatermarksResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> WatermarksResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return WatermarksResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        account_id: str,
        name: str | Omit = omit,
        opacity: float | Omit = omit,
        padding: float | Omit = omit,
        position: str | Omit = omit,
        scale: float | Omit = omit,
        url: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[Watermark]:
        """
        Creates watermark profiles using a single `HTTP POST multipart/form-data`
        request.

        Args:
          account_id: The account identifier tag.

          name: A short description of the watermark profile.

          opacity: The translucency of the image. A value of `0.0` makes the image completely
              transparent, and `1.0` makes the image completely opaque. Note that if the image
              is already semi-transparent, setting this to `1.0` will not make the image
              completely opaque.

          padding: The whitespace between the adjacent edges (determined by position) of the video
              and the image. `0.0` indicates no padding, and `1.0` indicates a fully padded
              video width or length, as determined by the algorithm.

          position: The location of the image. Valid positions are: `upperRight`, `upperLeft`,
              `lowerLeft`, `lowerRight`, and `center`. Note that `center` ignores the
              `padding` parameter.

          scale: The size of the image relative to the overall size of the video. This parameter
              will adapt to horizontal and vertical videos automatically. `0.0` indicates no
              scaling (use the size of the image as-is), and `1.0 `fills the entire video.

          url: URL of the watermark image to copy.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._post(
            path_template("/accounts/{account_id}/stream/watermarks", account_id=account_id),
            body=maybe_transform(
                {
                    "name": name,
                    "opacity": opacity,
                    "padding": padding,
                    "position": position,
                    "scale": scale,
                    "url": url,
                },
                watermark_create_params.WatermarkCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[Watermark]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[Watermark]], ResultWrapper[Watermark]),
        )

    def list(
        self,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncSinglePage[Watermark]:
        """
        Lists all watermark profiles for an account.

        Args:
          account_id: The account identifier tag.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get_api_list(
            path_template("/accounts/{account_id}/stream/watermarks", account_id=account_id),
            page=SyncSinglePage[Watermark],
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            model=Watermark,
        )

    def delete(
        self,
        identifier: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> str:
        """
        Deletes a watermark profile.

        Args:
          account_id: The account identifier tag.

          identifier: The unique identifier for a watermark profile.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not identifier:
            raise ValueError(f"Expected a non-empty value for `identifier` but received {identifier!r}")
        return self._delete(
            path_template(
                "/accounts/{account_id}/stream/watermarks/{identifier}", account_id=account_id, identifier=identifier
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[WatermarkDeleteResponse]]._unwrapper,
            ),
            cast_to=cast(Type[str], ResultWrapper[str]),
        )

    def get(
        self,
        identifier: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[Watermark]:
        """
        Retrieves details for a single watermark profile.

        Args:
          account_id: The account identifier tag.

          identifier: The unique identifier for a watermark profile.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not identifier:
            raise ValueError(f"Expected a non-empty value for `identifier` but received {identifier!r}")
        return self._get(
            path_template(
                "/accounts/{account_id}/stream/watermarks/{identifier}", account_id=account_id, identifier=identifier
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[Watermark]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[Watermark]], ResultWrapper[Watermark]),
        )


class AsyncWatermarksResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncWatermarksResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncWatermarksResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncWatermarksResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncWatermarksResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        account_id: str,
        name: str | Omit = omit,
        opacity: float | Omit = omit,
        padding: float | Omit = omit,
        position: str | Omit = omit,
        scale: float | Omit = omit,
        url: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[Watermark]:
        """
        Creates watermark profiles using a single `HTTP POST multipart/form-data`
        request.

        Args:
          account_id: The account identifier tag.

          name: A short description of the watermark profile.

          opacity: The translucency of the image. A value of `0.0` makes the image completely
              transparent, and `1.0` makes the image completely opaque. Note that if the image
              is already semi-transparent, setting this to `1.0` will not make the image
              completely opaque.

          padding: The whitespace between the adjacent edges (determined by position) of the video
              and the image. `0.0` indicates no padding, and `1.0` indicates a fully padded
              video width or length, as determined by the algorithm.

          position: The location of the image. Valid positions are: `upperRight`, `upperLeft`,
              `lowerLeft`, `lowerRight`, and `center`. Note that `center` ignores the
              `padding` parameter.

          scale: The size of the image relative to the overall size of the video. This parameter
              will adapt to horizontal and vertical videos automatically. `0.0` indicates no
              scaling (use the size of the image as-is), and `1.0 `fills the entire video.

          url: URL of the watermark image to copy.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._post(
            path_template("/accounts/{account_id}/stream/watermarks", account_id=account_id),
            body=await async_maybe_transform(
                {
                    "name": name,
                    "opacity": opacity,
                    "padding": padding,
                    "position": position,
                    "scale": scale,
                    "url": url,
                },
                watermark_create_params.WatermarkCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[Watermark]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[Watermark]], ResultWrapper[Watermark]),
        )

    def list(
        self,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[Watermark, AsyncSinglePage[Watermark]]:
        """
        Lists all watermark profiles for an account.

        Args:
          account_id: The account identifier tag.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get_api_list(
            path_template("/accounts/{account_id}/stream/watermarks", account_id=account_id),
            page=AsyncSinglePage[Watermark],
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            model=Watermark,
        )

    async def delete(
        self,
        identifier: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> str:
        """
        Deletes a watermark profile.

        Args:
          account_id: The account identifier tag.

          identifier: The unique identifier for a watermark profile.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not identifier:
            raise ValueError(f"Expected a non-empty value for `identifier` but received {identifier!r}")
        return await self._delete(
            path_template(
                "/accounts/{account_id}/stream/watermarks/{identifier}", account_id=account_id, identifier=identifier
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[WatermarkDeleteResponse]]._unwrapper,
            ),
            cast_to=cast(Type[str], ResultWrapper[str]),
        )

    async def get(
        self,
        identifier: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[Watermark]:
        """
        Retrieves details for a single watermark profile.

        Args:
          account_id: The account identifier tag.

          identifier: The unique identifier for a watermark profile.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not identifier:
            raise ValueError(f"Expected a non-empty value for `identifier` but received {identifier!r}")
        return await self._get(
            path_template(
                "/accounts/{account_id}/stream/watermarks/{identifier}", account_id=account_id, identifier=identifier
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[Watermark]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[Watermark]], ResultWrapper[Watermark]),
        )


class WatermarksResourceWithRawResponse:
    def __init__(self, watermarks: WatermarksResource) -> None:
        self._watermarks = watermarks

        self.create = to_raw_response_wrapper(
            watermarks.create,
        )
        self.list = to_raw_response_wrapper(
            watermarks.list,
        )
        self.delete = to_raw_response_wrapper(
            watermarks.delete,
        )
        self.get = to_raw_response_wrapper(
            watermarks.get,
        )


class AsyncWatermarksResourceWithRawResponse:
    def __init__(self, watermarks: AsyncWatermarksResource) -> None:
        self._watermarks = watermarks

        self.create = async_to_raw_response_wrapper(
            watermarks.create,
        )
        self.list = async_to_raw_response_wrapper(
            watermarks.list,
        )
        self.delete = async_to_raw_response_wrapper(
            watermarks.delete,
        )
        self.get = async_to_raw_response_wrapper(
            watermarks.get,
        )


class WatermarksResourceWithStreamingResponse:
    def __init__(self, watermarks: WatermarksResource) -> None:
        self._watermarks = watermarks

        self.create = to_streamed_response_wrapper(
            watermarks.create,
        )
        self.list = to_streamed_response_wrapper(
            watermarks.list,
        )
        self.delete = to_streamed_response_wrapper(
            watermarks.delete,
        )
        self.get = to_streamed_response_wrapper(
            watermarks.get,
        )


class AsyncWatermarksResourceWithStreamingResponse:
    def __init__(self, watermarks: AsyncWatermarksResource) -> None:
        self._watermarks = watermarks

        self.create = async_to_streamed_response_wrapper(
            watermarks.create,
        )
        self.list = async_to_streamed_response_wrapper(
            watermarks.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            watermarks.delete,
        )
        self.get = async_to_streamed_response_wrapper(
            watermarks.get,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["MeetingGetResponse", "Data", "Paging"]


class Data(BaseModel):
    id: str
    """ID of the meeting."""

    created_at: datetime
    """Timestamp the object was created at. The time is returned in ISO format."""

    updated_at: datetime
    """Timestamp the object was updated at. The time is returned in ISO format."""

    live_stream_on_start: Optional[bool] = None
    """Specifies if the meeting should start getting livestreamed on start."""

    persist_chat: Optional[bool] = None
    """Specifies if Chat within a meeting should persist for a week."""

    record_on_start: Optional[bool] = None
    """
    Specifies if the meeting should start getting recorded as soon as someone joins
    the meeting.
    """

    session_keep_alive_time_in_secs: Optional[float] = None
    """
    Time in seconds, for which a session remains active, after the last participant
    has left the meeting.
    """

    status: Optional[Literal["ACTIVE", "INACTIVE"]] = None
    """Whether the meeting is `ACTIVE` or `INACTIVE`.

    Users will not be able to join an `INACTIVE` meeting.
    """

    summarize_on_end: Optional[bool] = None
    """Automatically generate summary of meetings using transcripts.

    Requires Transcriptions to be enabled, and can be retrieved via Webhooks or
    summary API.
    """

    title: Optional[str] = None
    """Title of the meeting."""


class Paging(BaseModel):
    end_offset: float

    start_offset: float

    total_count: float


class MeetingGetResponse(BaseModel):
    data: List[Data]

    paging: Paging

    success: bool



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, Union, cast
from datetime import datetime
from typing_extensions import Literal

import httpx

from .token import (
    TokenResource,
    AsyncTokenResource,
    TokenResourceWithRawResponse,
    AsyncTokenResourceWithRawResponse,
    TokenResourceWithStreamingResponse,
    AsyncTokenResourceWithStreamingResponse,
)
from .failover import (
    FailoverResource,
    AsyncFailoverResource,
    FailoverResourceWithRawResponse,
    AsyncFailoverResourceWithRawResponse,
    FailoverResourceWithStreamingResponse,
    AsyncFailoverResourceWithStreamingResponse,
)
from ....._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ....._utils import path_template, maybe_transform, async_maybe_transform
from .connectors import (
    ConnectorsResource,
    AsyncConnectorsResource,
    ConnectorsResourceWithRawResponse,
    AsyncConnectorsResourceWithRawResponse,
    ConnectorsResourceWithStreamingResponse,
    AsyncConnectorsResourceWithStreamingResponse,
)
from ....._compat import cached_property
from .connections import (
    ConnectionsResource,
    AsyncConnectionsResource,
    ConnectionsResourceWithRawResponse,
    AsyncConnectionsResourceWithRawResponse,
    ConnectionsResourceWithStreamingResponse,
    AsyncConnectionsResourceWithStreamingResponse,
)
from ....._resource import SyncAPIResource, AsyncAPIResource
from ....._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ....._wrappers import ResultWrapper
from .....pagination import SyncV4PagePaginationArray, AsyncV4PagePaginationArray
from ....._base_client import AsyncPaginator, make_request_options
from .....types.zero_trust.tunnels import (
    warp_connector_edit_params,
    warp_connector_list_params,
    warp_connector_create_params,
)
from .....types.zero_trust.tunnels.warp_connector_get_response import WARPConnectorGetResponse
from .....types.zero_trust.tunnels.warp_connector_edit_response import WARPConnectorEditResponse
from .....types.zero_trust.tunnels.warp_connector_list_response import WARPConnectorListResponse
from .....types.zero_trust.tunnels.warp_connector_create_response import WARPConnectorCreateResponse
from .....types.zero_trust.tunnels.warp_connector_delete_response import WARPConnectorDeleteResponse

__all__ = ["WARPConnectorResource", "AsyncWARPConnectorResource"]


class WARPConnectorResource(SyncAPIResource):
    @cached_property
    def token(self) -> TokenResource:
        return TokenResource(self._client)

    @cached_property
    def connections(self) -> ConnectionsResource:
        return ConnectionsResource(self._client)

    @cached_property
    def connectors(self) -> ConnectorsResource:
        return ConnectorsResource(self._client)

    @cached_property
    def failover(self) -> FailoverResource:
        return FailoverResource(self._client)

    @cached_property
    def with_raw_response(self) -> WARPConnectorResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return WARPConnectorResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> WARPConnectorResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return WARPConnectorResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        account_id: str,
        name: str,
        ha: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> WARPConnectorCreateResponse:
        """
        Creates a new Warp Connector Tunnel in an account.

        Args:
          account_id: Cloudflare account ID

          name: A user-friendly name for a tunnel.

          ha: Indicates that the tunnel will be created to be highly available. If omitted,
              defaults to false.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._post(
            path_template("/accounts/{account_id}/warp_connector", account_id=account_id),
            body=maybe_transform(
                {
                    "name": name,
                    "ha": ha,
                },
                warp_connector_create_params.WARPConnectorCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[WARPConnectorCreateResponse]._unwrapper,
            ),
            cast_to=cast(Type[WARPConnectorCreateResponse], ResultWrapper[WARPConnectorCreateResponse]),
        )

    def list(
        self,
        *,
        account_id: str,
        exclude_prefix: str | Omit = omit,
        existed_at: str | Omit = omit,
        include_prefix: str | Omit = omit,
        is_deleted: bool | Omit = omit,
        name: str | Omit = omit,
        page: float | Omit = omit,
        per_page: float | Omit = omit,
        status: Literal["inactive", "degraded", "healthy", "down"] | Omit = omit,
        uuid: str | Omit = omit,
        was_active_at: Union[str, datetime] | Omit = omit,
        was_inactive_at: Union[str, datetime] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncV4PagePaginationArray[WARPConnectorListResponse]:
        """
        Lists and filters Warp Connector Tunnels in an account.

        Args:
          account_id: Cloudflare account ID

          existed_at: If provided, include only resources that were created (and not deleted) before
              this time. URL encoded.

          is_deleted: If `true`, only include deleted tunnels. If `false`, exclude deleted tunnels. If
              empty, all tunnels will be included.

          name: A user-friendly name for the tunnel.

          page: Page number of paginated results.

          per_page: Number of results to display.

          status: The status of the tunnel. Valid values are `inactive` (tunnel has never been
              run), `degraded` (tunnel is active and able to serve traffic but in an unhealthy
              state), `healthy` (tunnel is active and able to serve traffic), or `down`
              (tunnel can not serve traffic as it has no connections to the Cloudflare Edge).

          uuid: UUID of the tunnel.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get_api_list(
            path_template("/accounts/{account_id}/warp_connector", account_id=account_id),
            page=SyncV4PagePaginationArray[WARPConnectorListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "exclude_prefix": exclude_prefix,
                        "existed_at": existed_at,
                        "include_prefix": include_prefix,
                        "is_deleted": is_deleted,
                        "name": name,
                        "page": page,
                        "per_page": per_page,
                        "status": status,
                        "uuid": uuid,
                        "was_active_at": was_active_at,
                        "was_inactive_at": was_inactive_at,
                    },
                    warp_connector_list_params.WARPConnectorListParams,
                ),
            ),
            model=WARPConnectorListResponse,
        )

    def delete(
        self,
        tunnel_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> WARPConnectorDeleteResponse:
        """
        Deletes a Warp Connector Tunnel from an account.

        Args:
          account_id: Cloudflare account ID

          tunnel_id: UUID of the tunnel.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not tunnel_id:
            raise ValueError(f"Expected a non-empty value for `tunnel_id` but received {tunnel_id!r}")
        return self._delete(
            path_template(
                "/accounts/{account_id}/warp_connector/{tunnel_id}", account_id=account_id, tunnel_id=tunnel_id
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[WARPConnectorDeleteResponse]._unwrapper,
            ),
            cast_to=cast(Type[WARPConnectorDeleteResponse], ResultWrapper[WARPConnectorDeleteResponse]),
        )

    def edit(
        self,
        tunnel_id: str,
        *,
        account_id: str,
        name: str | Omit = omit,
        tunnel_secret: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> WARPConnectorEditResponse:
        """
        Updates an existing Warp Connector Tunnel.

        Args:
          account_id: Cloudflare account ID

          tunnel_id: UUID of the tunnel.

          name: A user-friendly name for a tunnel.

          tunnel_secret: Sets the password required to run a locally-managed tunnel. Must be at least 32
              bytes and encoded as a base64 string.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not tunnel_id:
            raise ValueError(f"Expected a non-empty value for `tunnel_id` but received {tunnel_id!r}")
        return self._patch(
            path_template(
                "/accounts/{account_id}/warp_connector/{tunnel_id}", account_id=account_id, tunnel_id=tunnel_id
            ),
            body=maybe_transform(
                {
                    "name": name,
                    "tunnel_secret": tunnel_secret,
                },
                warp_connector_edit_params.WARPConnectorEditParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[WARPConnectorEditResponse]._unwrapper,
            ),
            cast_to=cast(Type[WARPConnectorEditResponse], ResultWrapper[WARPConnectorEditResponse]),
        )

    def get(
        self,
        tunnel_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> WARPConnectorGetResponse:
        """
        Fetches a single Warp Connector Tunnel.

        Args:
          account_id: Cloudflare account ID

          tunnel_id: UUID of the tunnel.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not tunnel_id:
            raise ValueError(f"Expected a non-empty value for `tunnel_id` but received {tunnel_id!r}")
        return self._get(
            path_template(
                "/accounts/{account_id}/warp_connector/{tunnel_id}", account_id=account_id, tunnel_id=tunnel_id
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[WARPConnectorGetResponse]._unwrapper,
            ),
            cast_to=cast(Type[WARPConnectorGetResponse], ResultWrapper[WARPConnectorGetResponse]),
        )


class AsyncWARPConnectorResource(AsyncAPIResource):
    @cached_property
    def token(self) -> AsyncTokenResource:
        return AsyncTokenResource(self._client)

    @cached_property
    def connections(self) -> AsyncConnectionsResource:
        return AsyncConnectionsResource(self._client)

    @cached_property
    def connectors(self) -> AsyncConnectorsResource:
        return AsyncConnectorsResource(self._client)

    @cached_property
    def failover(self) -> AsyncFailoverResource:
        return AsyncFailoverResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncWARPConnectorResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncWARPConnectorResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncWARPConnectorResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncWARPConnectorResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        account_id: str,
        name: str,
        ha: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> WARPConnectorCreateResponse:
        """
        Creates a new Warp Connector Tunnel in an account.

        Args:
          account_id: Cloudflare account ID

          name: A user-friendly name for a tunnel.

          ha: Indicates that the tunnel will be created to be highly available. If omitted,
              defaults to false.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._post(
            path_template("/accounts/{account_id}/warp_connector", account_id=account_id),
            body=await async_maybe_transform(
                {
                    "name": name,
                    "ha": ha,
                },
                warp_connector_create_params.WARPConnectorCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[WARPConnectorCreateResponse]._unwrapper,
            ),
            cast_to=cast(Type[WARPConnectorCreateResponse], ResultWrapper[WARPConnectorCreateResponse]),
        )

    def list(
        self,
        *,
        account_id: str,
        exclude_prefix: str | Omit = omit,
        existed_at: str | Omit = omit,
        include_prefix: str | Omit = omit,
        is_deleted: bool | Omit = omit,
        name: str | Omit = omit,
        page: float | Omit = omit,
        per_page: float | Omit = omit,
        status: Literal["inactive", "degraded", "healthy", "down"] | Omit = omit,
        uuid: str | Omit = omit,
        was_active_at: Union[str, datetime] | Omit = omit,
        was_inactive_at: Union[str, datetime] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[WARPConnectorListResponse, AsyncV4PagePaginationArray[WARPConnectorListResponse]]:
        """
        Lists and filters Warp Connector Tunnels in an account.

        Args:
          account_id: Cloudflare account ID

          existed_at: If provided, include only resources that were created (and not deleted) before
              this time. URL encoded.

          is_deleted: If `true`, only include deleted tunnels. If `false`, exclude deleted tunnels. If
              empty, all tunnels will be included.

          name: A user-friendly name for the tunnel.

          page: Page number of paginated results.

          per_page: Number of results to display.

          status: The status of the tunnel. Valid values are `inactive` (tunnel has never been
              run), `degraded` (tunnel is active and able to serve traffic but in an unhealthy
              state), `healthy` (tunnel is active and able to serve traffic), or `down`
              (tunnel can not serve traffic as it has no connections to the Cloudflare Edge).

          uuid: UUID of the tunnel.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get_api_list(
            path_template("/accounts/{account_id}/warp_connector", account_id=account_id),
            page=AsyncV4PagePaginationArray[WARPConnectorListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "exclude_prefix": exclude_prefix,
                        "existed_at": existed_at,
                        "include_prefix": include_prefix,
                        "is_deleted": is_deleted,
                        "name": name,
                        "page": page,
                        "per_page": per_page,
                        "status": status,
                        "uuid": uuid,
                        "was_active_at": was_active_at,
                        "was_inactive_at": was_inactive_at,
                    },
                    warp_connector_list_params.WARPConnectorListParams,
                ),
            ),
            model=WARPConnectorListResponse,
        )

    async def delete(
        self,
        tunnel_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> WARPConnectorDeleteResponse:
        """
        Deletes a Warp Connector Tunnel from an account.

        Args:
          account_id: Cloudflare account ID

          tunnel_id: UUID of the tunnel.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not tunnel_id:
            raise ValueError(f"Expected a non-empty value for `tunnel_id` but received {tunnel_id!r}")
        return await self._delete(
            path_template(
                "/accounts/{account_id}/warp_connector/{tunnel_id}", account_id=account_id, tunnel_id=tunnel_id
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[WARPConnectorDeleteResponse]._unwrapper,
            ),
            cast_to=cast(Type[WARPConnectorDeleteResponse], ResultWrapper[WARPConnectorDeleteResponse]),
        )

    async def edit(
        self,
        tunnel_id: str,
        *,
        account_id: str,
        name: str | Omit = omit,
        tunnel_secret: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> WARPConnectorEditResponse:
        """
        Updates an existing Warp Connector Tunnel.

        Args:
          account_id: Cloudflare account ID

          tunnel_id: UUID of the tunnel.

          name: A user-friendly name for a tunnel.

          tunnel_secret: Sets the password required to run a locally-managed tunnel. Must be at least 32
              bytes and encoded as a base64 string.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not tunnel_id:
            raise ValueError(f"Expected a non-empty value for `tunnel_id` but received {tunnel_id!r}")
        return await self._patch(
            path_template(
                "/accounts/{account_id}/warp_connector/{tunnel_id}", account_id=account_id, tunnel_id=tunnel_id
            ),
            body=await async_maybe_transform(
                {
                    "name": name,
                    "tunnel_secret": tunnel_secret,
                },
                warp_connector_edit_params.WARPConnectorEditParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[WARPConnectorEditResponse]._unwrapper,
            ),
            cast_to=cast(Type[WARPConnectorEditResponse], ResultWrapper[WARPConnectorEditResponse]),
        )

    async def get(
        self,
        tunnel_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> WARPConnectorGetResponse:
        """
        Fetches a single Warp Connector Tunnel.

        Args:
          account_id: Cloudflare account ID

          tunnel_id: UUID of the tunnel.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not tunnel_id:
            raise ValueError(f"Expected a non-empty value for `tunnel_id` but received {tunnel_id!r}")
        return await self._get(
            path_template(
                "/accounts/{account_id}/warp_connector/{tunnel_id}", account_id=account_id, tunnel_id=tunnel_id
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[WARPConnectorGetResponse]._unwrapper,
            ),
            cast_to=cast(Type[WARPConnectorGetResponse], ResultWrapper[WARPConnectorGetResponse]),
        )


class WARPConnectorResourceWithRawResponse:
    def __init__(self, warp_connector: WARPConnectorResource) -> None:
        self._warp_connector = warp_connector

        self.create = to_raw_response_wrapper(
            warp_connector.create,
        )
        self.list = to_raw_response_wrapper(
            warp_connector.list,
        )
        self.delete = to_raw_response_wrapper(
            warp_connector.delete,
        )
        self.edit = to_raw_response_wrapper(
            warp_connector.edit,
        )
        self.get = to_raw_response_wrapper(
            warp_connector.get,
        )

    @cached_property
    def token(self) -> TokenResourceWithRawResponse:
        return TokenResourceWithRawResponse(self._warp_connector.token)

    @cached_property
    def connections(self) -> ConnectionsResourceWithRawResponse:
        return ConnectionsResourceWithRawResponse(self._warp_connector.connections)

    @cached_property
    def connectors(self) -> ConnectorsResourceWithRawResponse:
        return ConnectorsResourceWithRawResponse(self._warp_connector.connectors)

    @cached_property
    def failover(self) -> FailoverResourceWithRawResponse:
        return FailoverResourceWithRawResponse(self._warp_connector.failover)


class AsyncWARPConnectorResourceWithRawResponse:
    def __init__(self, warp_connector: AsyncWARPConnectorResource) -> None:
        self._warp_connector = warp_connector

        self.create = async_to_raw_response_wrapper(
            warp_connector.create,
        )
        self.list = async_to_raw_response_wrapper(
            warp_connector.list,
        )
        self.delete = async_to_raw_response_wrapper(
            warp_connector.delete,
        )
        self.edit = async_to_raw_response_wrapper(
            warp_connector.edit,
        )
        self.get = async_to_raw_response_wrapper(
            warp_connector.get,
        )

    @cached_property
    def token(self) -> AsyncTokenResourceWithRawResponse:
        return AsyncTokenResourceWithRawResponse(self._warp_connector.token)

    @cached_property
    def connections(self) -> AsyncConnectionsResourceWithRawResponse:
        return AsyncConnectionsResourceWithRawResponse(self._warp_connector.connections)

    @cached_property
    def connectors(self) -> AsyncConnectorsResourceWithRawResponse:
        return AsyncConnectorsResourceWithRawResponse(self._warp_connector.connectors)

    @cached_property
    def failover(self) -> AsyncFailoverResourceWithRawResponse:
        return AsyncFailoverResourceWithRawResponse(self._warp_connector.failover)


class WARPConnectorResourceWithStreamingResponse:
    def __init__(self, warp_connector: WARPConnectorResource) -> None:
        self._warp_connector = warp_connector

        self.create = to_streamed_response_wrapper(
            warp_connector.create,
        )
        self.list = to_streamed_response_wrapper(
            warp_connector.list,
        )
        self.delete = to_streamed_response_wrapper(
            warp_connector.delete,
        )
        self.edit = to_streamed_response_wrapper(
            warp_connector.edit,
        )
        self.get = to_streamed_response_wrapper(
            warp_connector.get,
        )

    @cached_property
    def token(self) -> TokenResourceWithStreamingResponse:
        return TokenResourceWithStreamingResponse(self._warp_connector.token)

    @cached_property
    def connections(self) -> ConnectionsResourceWithStreamingResponse:
        return ConnectionsResourceWithStreamingResponse(self._warp_connector.connections)

    @cached_property
    def connectors(self) -> ConnectorsResourceWithStreamingResponse:
        return ConnectorsResourceWithStreamingResponse(self._warp_connector.connectors)

    @cached_property
    def failover(self) -> FailoverResourceWithStreamingResponse:
        return FailoverResourceWithStreamingResponse(self._warp_connector.failover)


class AsyncWARPConnectorResourceWithStreamingResponse:
    def __init__(self, warp_connector: AsyncWARPConnectorResource) -> None:
        self._warp_connector = warp_connector

        self.create = async_to_streamed_response_wrapper(
            warp_connector.create,
        )
        self.list = async_to_streamed_response_wrapper(
            warp_connector.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            warp_connector.delete,
        )
        self.edit = async_to_streamed_response_wrapper(
            warp_connector.edit,
        )
        self.get = async_to_streamed_response_wrapper(
            warp_connector.get,
        )

    @cached_property
    def token(self) -> AsyncTokenResourceWithStreamingResponse:
        return AsyncTokenResourceWithStreamingResponse(self._warp_connector.token)

    @cached_property
    def connections(self) -> AsyncConnectionsResourceWithStreamingResponse:
        return AsyncConnectionsResourceWithStreamingResponse(self._warp_connector.connections)

    @cached_property
    def connectors(self) -> AsyncConnectorsResourceWithStreamingResponse:
        return AsyncConnectorsResourceWithStreamingResponse(self._warp_connector.connectors)

    @cached_property
    def failover(self) -> AsyncFailoverResourceWithStreamingResponse:
        return AsyncFailoverResourceWithStreamingResponse(self._warp_connector.failover)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

from ..._types import FileTypes, SequenceNotStr

__all__ = ["ToMarkdownTransformParams", "File"]


class ToMarkdownTransformParams(TypedDict, total=False):
    account_id: Required[str]

    file: Required[File]


class File(TypedDict, total=False):
    files: Required[SequenceNotStr[FileTypes]]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, Annotated, TypedDict

from ...._utils import PropertyInfo

__all__ = ["MemberListParams"]


class MemberListParams(TypedDict, total=False):
    account_id: Required[str]
    """Account identifier tag."""

    direction: Literal["asc", "desc"]
    """The sort order of returned user group members by email."""

    fuzzy_email: Annotated[str, PropertyInfo(alias="fuzzyEmail")]
    """A string used for filtering members by partial email match."""

    page: float
    """Page number of paginated results."""

    per_page: float
    """Maximum number of results per page."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from ...._models import BaseModel
from .doh_endpoint import DOHEndpoint
from .dot_endpoint import DOTEndpoint
from .ipv4_endpoint import IPV4Endpoint
from .ipv6_endpoint import IPV6Endpoint

__all__ = ["Endpoint"]


class Endpoint(BaseModel):
    """Configure the destination endpoints for this location."""

    doh: DOHEndpoint

    dot: DOTEndpoint

    ipv4: IPV4Endpoint

    ipv6: IPV6Endpoint



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Union, Optional
from datetime import datetime
from typing_extensions import Literal, TypeAlias

from pydantic import Field as FieldInfo

from ..._models import BaseModel

__all__ = [
    "SubscriptionGetResponse",
    "Destination",
    "Source",
    "SourceMqEventSourceImages",
    "SourceMqEventSourceKV",
    "SourceMqEventSourceR2",
    "SourceMqEventSourceSuperSlurper",
    "SourceMqEventSourceVectorize",
    "SourceMqEventSourceWorkersAIModel",
    "SourceMqEventSourceWorkersBuildsWorker",
    "SourceMqEventSourceWorkflowsWorkflow",
]


class Destination(BaseModel):
    """Destination configuration for the subscription"""

    queue_id: str
    """ID of the target queue"""

    type: Literal["queues.queue"]
    """Type of destination"""


class SourceMqEventSourceImages(BaseModel):
    type: Optional[Literal["images"]] = None
    """Type of source"""


class SourceMqEventSourceKV(BaseModel):
    type: Optional[Literal["kv"]] = None
    """Type of source"""


class SourceMqEventSourceR2(BaseModel):
    type: Optional[Literal["r2"]] = None
    """Type of source"""


class SourceMqEventSourceSuperSlurper(BaseModel):
    type: Optional[Literal["superSlurper"]] = None
    """Type of source"""


class SourceMqEventSourceVectorize(BaseModel):
    type: Optional[Literal["vectorize"]] = None
    """Type of source"""


class SourceMqEventSourceWorkersAIModel(BaseModel):
    ai_model_name: Optional[str] = FieldInfo(alias="model_name", default=None)
    """Name of the Workers AI model"""

    type: Optional[Literal["workersAi.model"]] = None
    """Type of source"""


class SourceMqEventSourceWorkersBuildsWorker(BaseModel):
    type: Optional[Literal["workersBuilds.worker"]] = None
    """Type of source"""

    worker_name: Optional[str] = None
    """Name of the worker"""


class SourceMqEventSourceWorkflowsWorkflow(BaseModel):
    type: Optional[Literal["workflows.workflow"]] = None
    """Type of source"""

    workflow_name: Optional[str] = None
    """Name of the workflow"""


Source: TypeAlias = Union[
    SourceMqEventSourceImages,
    SourceMqEventSourceKV,
    SourceMqEventSourceR2,
    SourceMqEventSourceSuperSlurper,
    SourceMqEventSourceVectorize,
    SourceMqEventSourceWorkersAIModel,
    SourceMqEventSourceWorkersBuildsWorker,
    SourceMqEventSourceWorkflowsWorkflow,
]


class SubscriptionGetResponse(BaseModel):
    id: str
    """Unique identifier for the subscription"""

    created_at: datetime
    """When the subscription was created"""

    destination: Destination
    """Destination configuration for the subscription"""

    enabled: bool
    """Whether the subscription is active"""

    events: List[str]
    """List of event types this subscription handles"""

    modified_at: datetime
    """When the subscription was last modified"""

    name: str
    """Name of the subscription"""

    source: Source
    """Source configuration for the subscription"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, Optional, cast
from typing_extensions import Literal

import httpx

from ...._types import Body, Omit, Query, Headers, NotGiven, SequenceNotStr, omit, not_given
from ...._utils import path_template, maybe_transform, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._wrappers import ResultWrapper
from ....pagination import SyncCursorPagination, AsyncCursorPagination
from ...._base_client import AsyncPaginator, make_request_options
from ....types.zero_trust.devices import device_get_params, device_list_params
from ....types.zero_trust.devices.device_get_response import DeviceGetResponse
from ....types.zero_trust.devices.device_list_response import DeviceListResponse

__all__ = ["DevicesResource", "AsyncDevicesResource"]


class DevicesResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> DevicesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return DevicesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> DevicesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return DevicesResourceWithStreamingResponse(self)

    def list(
        self,
        *,
        account_id: str,
        id: SequenceNotStr[str] | Omit = omit,
        active_registrations: Literal["include", "only", "exclude"] | Omit = omit,
        cursor: str | Omit = omit,
        include: str | Omit = omit,
        last_seen_user: device_list_params.LastSeenUser | Omit = omit,
        per_page: int | Omit = omit,
        search: str | Omit = omit,
        seen_after: str | Omit = omit,
        seen_before: str | Omit = omit,
        sort_by: Literal[
            "name", "id", "client_version", "last_seen_user.email", "last_seen_at", "active_registrations", "created_at"
        ]
        | Omit = omit,
        sort_order: Literal["asc", "desc"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncCursorPagination[DeviceListResponse]:
        """
        Lists WARP devices.

        Args:
          id: Filter by a one or more device IDs.

          active_registrations: Include or exclude devices with active registrations. The default is "only" -
              return only devices with active registrations.

          cursor: Opaque token indicating the starting position when requesting the next set of
              records. A cursor value can be obtained from the result_info.cursor field in the
              response.

          include: Comma-separated list of additional information that should be included in the
              device response. Supported values are: "last_seen_registration.policy".

          per_page: The maximum number of devices to return in a single response.

          search: Search by device details.

          seen_after: Filter by the last_seen timestamp - returns only devices last seen after this
              timestamp.

          seen_before: Filter by the last_seen timestamp - returns only devices last seen before this
              timestamp.

          sort_by: The device field to order results by.

          sort_order: Sort direction.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get_api_list(
            path_template("/accounts/{account_id}/devices/physical-devices", account_id=account_id),
            page=SyncCursorPagination[DeviceListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "id": id,
                        "active_registrations": active_registrations,
                        "cursor": cursor,
                        "include": include,
                        "last_seen_user": last_seen_user,
                        "per_page": per_page,
                        "search": search,
                        "seen_after": seen_after,
                        "seen_before": seen_before,
                        "sort_by": sort_by,
                        "sort_order": sort_order,
                    },
                    device_list_params.DeviceListParams,
                ),
            ),
            model=DeviceListResponse,
        )

    def delete(
        self,
        device_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> object:
        """
        Deletes a WARP device.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not device_id:
            raise ValueError(f"Expected a non-empty value for `device_id` but received {device_id!r}")
        return self._delete(
            path_template(
                "/accounts/{account_id}/devices/physical-devices/{device_id}",
                account_id=account_id,
                device_id=device_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[object]]._unwrapper,
            ),
            cast_to=cast(Type[object], ResultWrapper[object]),
        )

    def get(
        self,
        device_id: str,
        *,
        account_id: str,
        include: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> DeviceGetResponse:
        """
        Fetches a single WARP device.

        Args:
          include: Comma-separated list of additional information that should be included in the
              device response. Supported values are: "last_seen_registration.policy".

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not device_id:
            raise ValueError(f"Expected a non-empty value for `device_id` but received {device_id!r}")
        return self._get(
            path_template(
                "/accounts/{account_id}/devices/physical-devices/{device_id}",
                account_id=account_id,
                device_id=device_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"include": include}, device_get_params.DeviceGetParams),
                post_parser=ResultWrapper[DeviceGetResponse]._unwrapper,
            ),
            cast_to=cast(Type[DeviceGetResponse], ResultWrapper[DeviceGetResponse]),
        )

    def revoke(
        self,
        device_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> object:
        """
        Revokes all WARP registrations associated with the specified device.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not device_id:
            raise ValueError(f"Expected a non-empty value for `device_id` but received {device_id!r}")
        return self._post(
            path_template(
                "/accounts/{account_id}/devices/physical-devices/{device_id}/revoke",
                account_id=account_id,
                device_id=device_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[object]]._unwrapper,
            ),
            cast_to=cast(Type[object], ResultWrapper[object]),
        )


class AsyncDevicesResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncDevicesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncDevicesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncDevicesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncDevicesResourceWithStreamingResponse(self)

    def list(
        self,
        *,
        account_id: str,
        id: SequenceNotStr[str] | Omit = omit,
        active_registrations: Literal["include", "only", "exclude"] | Omit = omit,
        cursor: str | Omit = omit,
        include: str | Omit = omit,
        last_seen_user: device_list_params.LastSeenUser | Omit = omit,
        per_page: int | Omit = omit,
        search: str | Omit = omit,
        seen_after: str | Omit = omit,
        seen_before: str | Omit = omit,
        sort_by: Literal[
            "name", "id", "client_version", "last_seen_user.email", "last_seen_at", "active_registrations", "created_at"
        ]
        | Omit = omit,
        sort_order: Literal["asc", "desc"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[DeviceListResponse, AsyncCursorPagination[DeviceListResponse]]:
        """
        Lists WARP devices.

        Args:
          id: Filter by a one or more device IDs.

          active_registrations: Include or exclude devices with active registrations. The default is "only" -
              return only devices with active registrations.

          cursor: Opaque token indicating the starting position when requesting the next set of
              records. A cursor value can be obtained from the result_info.cursor field in the
              response.

          include: Comma-separated list of additional information that should be included in the
              device response. Supported values are: "last_seen_registration.policy".

          per_page: The maximum number of devices to return in a single response.

          search: Search by device details.

          seen_after: Filter by the last_seen timestamp - returns only devices last seen after this
              timestamp.

          seen_before: Filter by the last_seen timestamp - returns only devices last seen before this
              timestamp.

          sort_by: The device field to order results by.

          sort_order: Sort direction.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get_api_list(
            path_template("/accounts/{account_id}/devices/physical-devices", account_id=account_id),
            page=AsyncCursorPagination[DeviceListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "id": id,
                        "active_registrations": active_registrations,
                        "cursor": cursor,
                        "include": include,
                        "last_seen_user": last_seen_user,
                        "per_page": per_page,
                        "search": search,
                        "seen_after": seen_after,
                        "seen_before": seen_before,
                        "sort_by": sort_by,
                        "sort_order": sort_order,
                    },
                    device_list_params.DeviceListParams,
                ),
            ),
            model=DeviceListResponse,
        )

    async def delete(
        self,
        device_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> object:
        """
        Deletes a WARP device.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not device_id:
            raise ValueError(f"Expected a non-empty value for `device_id` but received {device_id!r}")
        return await self._delete(
            path_template(
                "/accounts/{account_id}/devices/physical-devices/{device_id}",
                account_id=account_id,
                device_id=device_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[object]]._unwrapper,
            ),
            cast_to=cast(Type[object], ResultWrapper[object]),
        )

    async def get(
        self,
        device_id: str,
        *,
        account_id: str,
        include: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> DeviceGetResponse:
        """
        Fetches a single WARP device.

        Args:
          include: Comma-separated list of additional information that should be included in the
              device response. Supported values are: "last_seen_registration.policy".

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not device_id:
            raise ValueError(f"Expected a non-empty value for `device_id` but received {device_id!r}")
        return await self._get(
            path_template(
                "/accounts/{account_id}/devices/physical-devices/{device_id}",
                account_id=account_id,
                device_id=device_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform({"include": include}, device_get_params.DeviceGetParams),
                post_parser=ResultWrapper[DeviceGetResponse]._unwrapper,
            ),
            cast_to=cast(Type[DeviceGetResponse], ResultWrapper[DeviceGetResponse]),
        )

    async def revoke(
        self,
        device_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> object:
        """
        Revokes all WARP registrations associated with the specified device.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not device_id:
            raise ValueError(f"Expected a non-empty value for `device_id` but received {device_id!r}")
        return await self._post(
            path_template(
                "/accounts/{account_id}/devices/physical-devices/{device_id}/revoke",
                account_id=account_id,
                device_id=device_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[object]]._unwrapper,
            ),
            cast_to=cast(Type[object], ResultWrapper[object]),
        )


class DevicesResourceWithRawResponse:
    def __init__(self, devices: DevicesResource) -> None:
        self._devices = devices

        self.list = to_raw_response_wrapper(
            devices.list,
        )
        self.delete = to_raw_response_wrapper(
            devices.delete,
        )
        self.get = to_raw_response_wrapper(
            devices.get,
        )
        self.revoke = to_raw_response_wrapper(
            devices.revoke,
        )


class AsyncDevicesResourceWithRawResponse:
    def __init__(self, devices: AsyncDevicesResource) -> None:
        self._devices = devices

        self.list = async_to_raw_response_wrapper(
            devices.list,
        )
        self.delete = async_to_raw_response_wrapper(
            devices.delete,
        )
        self.get = async_to_raw_response_wrapper(
            devices.get,
        )
        self.revoke = async_to_raw_response_wrapper(
            devices.revoke,
        )


class DevicesResourceWithStreamingResponse:
    def __init__(self, devices: DevicesResource) -> None:
        self._devices = devices

        self.list = to_streamed_response_wrapper(
            devices.list,
        )
        self.delete = to_streamed_response_wrapper(
            devices.delete,
        )
        self.get = to_streamed_response_wrapper(
            devices.get,
        )
        self.revoke = to_streamed_response_wrapper(
            devices.revoke,
        )


class AsyncDevicesResourceWithStreamingResponse:
    def __init__(self, devices: AsyncDevicesResource) -> None:
        self._devices = devices

        self.list = async_to_streamed_response_wrapper(
            devices.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            devices.delete,
        )
        self.get = async_to_streamed_response_wrapper(
            devices.get,
        )
        self.revoke = async_to_streamed_response_wrapper(
            devices.revoke,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, Annotated, TypedDict

from ...._utils import PropertyInfo

__all__ = ["VariantEditParams", "Options"]


class VariantEditParams(TypedDict, total=False):
    account_id: Required[str]
    """Account identifier tag."""

    options: Required[Options]
    """Allows you to define image resizing sizes for different use cases."""

    never_require_signed_urls: Annotated[bool, PropertyInfo(alias="neverRequireSignedURLs")]
    """
    Indicates whether the variant can access an image without a signature,
    regardless of image access control.
    """


class Options(TypedDict, total=False):
    """Allows you to define image resizing sizes for different use cases."""

    fit: Required[Literal["scale-down", "contain", "cover", "crop", "pad"]]
    """
    The fit property describes how the width and height dimensions should be
    interpreted.
    """

    height: Required[float]
    """Maximum height in image pixels."""

    metadata: Required[Literal["keep", "copyright", "none"]]
    """What EXIF data should be preserved in the output image."""

    width: Required[float]
    """Maximum width in image pixels."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from pydantic import Field as FieldInfo

from ..._models import BaseModel

__all__ = [
    "ScanHARResponse",
    "Log",
    "LogCreator",
    "LogEntry",
    "LogEntryRequest",
    "LogEntryRequestHeader",
    "LogEntryResponse",
    "LogEntryResponseContent",
    "LogEntryResponseHeader",
    "LogPage",
    "LogPagePageTimings",
]


class LogCreator(BaseModel):
    comment: str

    name: str

    version: str


class LogEntryRequestHeader(BaseModel):
    name: str

    value: str


class LogEntryRequest(BaseModel):
    body_size: float = FieldInfo(alias="bodySize")

    headers: List[LogEntryRequestHeader]

    headers_size: float = FieldInfo(alias="headersSize")

    http_version: str = FieldInfo(alias="httpVersion")

    method: str

    url: str


class LogEntryResponseContent(BaseModel):
    mime_type: str = FieldInfo(alias="mimeType")

    size: float

    compression: Optional[int] = None


class LogEntryResponseHeader(BaseModel):
    name: str

    value: str


class LogEntryResponse(BaseModel):
    transfer_size: float = FieldInfo(alias="_transferSize")

    body_size: float = FieldInfo(alias="bodySize")

    content: LogEntryResponseContent

    headers: List[LogEntryResponseHeader]

    headers_size: float = FieldInfo(alias="headersSize")

    http_version: str = FieldInfo(alias="httpVersion")

    redirect_url: str = FieldInfo(alias="redirectURL")

    status: float

    status_text: str = FieldInfo(alias="statusText")


class LogEntry(BaseModel):
    initial_priority: str = FieldInfo(alias="_initialPriority")

    initiator_type: str = FieldInfo(alias="_initiator_type")

    priority: str = FieldInfo(alias="_priority")

    request_id: str = FieldInfo(alias="_requestId")

    request_time: float = FieldInfo(alias="_requestTime")

    resource_type: str = FieldInfo(alias="_resourceType")

    cache: object

    connection: str

    pageref: str

    request: LogEntryRequest

    response: LogEntryResponse

    server_ip_address: str = FieldInfo(alias="serverIPAddress")

    started_date_time: str = FieldInfo(alias="startedDateTime")

    time: float


class LogPagePageTimings(BaseModel):
    on_content_load: float = FieldInfo(alias="onContentLoad")

    on_load: float = FieldInfo(alias="onLoad")


class LogPage(BaseModel):
    id: str

    page_timings: LogPagePageTimings = FieldInfo(alias="pageTimings")

    started_date_time: str = FieldInfo(alias="startedDateTime")

    title: str


class Log(BaseModel):
    creator: LogCreator

    entries: List[LogEntry]

    pages: List[LogPage]

    version: str


class ScanHARResponse(BaseModel):
    log: Log



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from ..._models import BaseModel

__all__ = ["Tunnel"]


class Tunnel(BaseModel):
    """Configuration for using Keyless SSL through a Cloudflare Tunnel"""

    private_ip: str
    """Private IP of the Key Server Host"""

    vnet_id: str
    """Cloudflare Tunnel Virtual Network ID"""
